@@ -27,7 +27,8 @@ else
 		export ACLOCAL_FLAGS="-I /usr/share/glade/gnome"
 		. $gnome_autogen $*
 	else
-		echo "Could not find gnome-autogen.sh"
+		echo "Could not find autogen.sh"
+		echo "Make sure glade is installed"
 	fi
 fi
 
