@@ -609,3 +609,10 @@ blowfish_get_info( int algo, size_t *keylen,
 	return "BLOWFISH";
     return NULL;
 }
+
+size_t
+blowfish_contextsize(void)
+{
+	return(sizeof(BLOWFISH_context));
+}
+
