@@ -549,3 +549,17 @@ on_button_import_cancel_clicked        (GtkButton       *button,
 
 }
 
+
+void
+on_current_password_file1_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gchar *dia_str;
+
+	dia_str = g_strdup_printf("Current Location of Password file:\n"
+		"%s", glb_filename);
+	gnome_ok_dialog( dia_str );
+	g_free(dia_str);
+
+}
+
