@@ -26,9 +26,10 @@
 #include "interface.h"
 #include "support.h"
 #include "fpm_crypt.h"
+#include <fcntl.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-
+#include <errno.h>
 
 /* GLOBAL VARIABLES */
 GtkWidget* glb_win_app;   /* The main window for the application   */
@@ -55,6 +56,9 @@ static gchar* glb_filename_lock_pid; /* The location of the pasword loc kfile. *
 #endif
 
 static int lock_fd = -1;
+#if defined(sun) && defined(__SVR4)
+struct flock lck;
+#endif
 
 void* old_context;
 void* new_context;
@@ -91,25 +95,27 @@ gboolean glb_quitting=FALSE;
 gchar* glb_last_category;
 
 void
-lock_fpm_file(void)
+lock_fpm_file(gchar* glb_filename)
 {
 	gchar *dia_str;
 	gchar *cmd;
-	char fname[1024];
+	gchar* pwd_dir_name = NULL;
 	struct stat fs_stat;
 
-	sprintf(fname, "%s/.fpm", g_get_home_dir());
+        /* iif g_dirname goes away in one of the next Glib releases
+         * we'll need to replace it by g_path_get_dirname */
+        pwd_dir_name = g_dirname(glb_filename);
 
-	/* Need the .fpm directory to create a lock. This is called
-	   when fpm is being used for the first time.
-	*/
+	/* Need the directory of the password file to create a lock.
+         * This is called when fpm is being used for the first time.
+	 */
 
-	if ( stat(fname, &fs_stat) )
+	if ( stat(pwd_dir_name, &fs_stat) )
 	{
-		cmd = g_strdup_printf("mkdir %s/.fpm", g_get_home_dir());
+		cmd = g_strdup_printf("mkdir %s", pwd_dir_name);
 		gnome_execute_shell(NULL, cmd);
 		g_free(cmd);
-		cmd = g_strdup_printf("chmod 0700 %s/.fpm", g_get_home_dir());
+		cmd = g_strdup_printf("chmod 0700 %s", pwd_dir_name);
 		gnome_execute_shell(NULL, cmd);
 		g_free(cmd);
 		/* Wait for the shell commands to finish */
@@ -136,7 +142,16 @@ lock_fpm_file(void)
 	   system. This will work on nfs file on the same system. 
 	*/
 
+#if defined(sun) && defined(__SVR4)
+	lck.l_type = F_WRLCK;	/* setting a write lock */
+	lck.l_whence = 0;	/* offset l_start from beginning of file */
+	lck.l_start = (off_t)0;	
+	lck.l_len = (off_t)0;	/* until the end of the file */
+
+	if ( (fcntl(lock_fd, F_SETLK ) <0) && (errno == EAGAIN || errno == EACCES) )
+#else
 	if ( flock(lock_fd, LOCK_EX | LOCK_NB) )
+#endif
 	{
 		dia_str = g_strdup_printf("Could not lock %s\nFile already locked.", 
 			glb_filename_lock);
@@ -179,7 +194,7 @@ lock_fpm_file(void)
 	{
 		dia_str = g_strdup_printf("Lock file %s exists.\nIf you know "
 			"this to be an error, clean up the\nlock files in"
-			"your ~/.fpm directory.", glb_filename_lock );
+			"the directory %s.", glb_filename_lock, pwd_dir_name);
 		glb_win_misc = gnome_error_dialog(dia_str);
 		gnome_dialog_run(GNOME_DIALOG(glb_win_misc));
 		unlink(glb_filename_lock_pid);
@@ -399,7 +414,7 @@ fpm_set_entry(GtkWidget* win, gchar* entry_name, char* text)
 }
 
 void
-fpm_init(void)
+fpm_init(char* opt_file_name)
 {
 #ifdef FPM_LINK_LOCK
 	char hostname[256];
@@ -408,7 +423,10 @@ fpm_init(void)
   glb_launcher_string_list=NULL;
   glb_win_app = create_app_safe();
 
-  glb_filename=g_strdup_printf("%s/.fpm/fpm", g_get_home_dir());
+  if( opt_file_name )
+    glb_filename = opt_file_name;
+  else
+    glb_filename=g_strdup_printf("%s/.fpm/fpm", g_get_home_dir());
 
   /* Lock filename. Should be the same directory as the password file. */
 
@@ -432,7 +450,7 @@ fpm_init(void)
 
   /* Lock the password file */
 
-    lock_fpm_file();
+    lock_fpm_file(glb_filename);
 
  /* Now done in load passfile 
   * fpm_init_launchers();
@@ -648,7 +666,6 @@ void fpm_set_password()
 {
   GtkEntry *entry;
   gchar *pw1, *pw2;
-  gchar *cmd;
   byte *md5val, *prehash;
 
   entry=GTK_ENTRY(lookup_widget(glb_win_misc, "entry_cpw1"));
@@ -667,13 +684,6 @@ void fpm_set_password()
     return;
   }
 
-  cmd = g_strdup_printf("mkdir %s/.fpm", g_get_home_dir());
-  gnome_execute_shell(NULL, cmd);
-  g_free(cmd);
-  cmd = g_strdup_printf("chmod 0700 %s/.fpm", g_get_home_dir());
-  gnome_execute_shell(NULL, cmd);
-  g_free(cmd);
-
   if(old_context == NULL)
   {
     /* This is the first time we are running the app.*/
