@@ -18,4 +18,6 @@
  * passfile.h
  */
 void passfile_save(gchar* file_name);
+void passfile_export(gchar* file_name);
+void passfile_import(gchar *fname);
 gint passfile_load(gchar* file_name);
